/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   cmd.c
 * Class:       Cmd
 * Created:     0.1, 970430, Per-Erik Forssn
 * Revised:     1.1, 980219, Torbjrn Frostemark
 * Revised:     1.2, 980825, Christian Jnsson
 * 
 * Description: Command processing functions, one per command.
 * Remarks:     In alphabetical order.
 *
 * Private Metods:     None.
 * Private Attributes: None.
 *----------------------------------------------------------------------
 */

#include        "cmd.h"
#include        "cmdsubr.h"
#include        "getput.h"
#include	"clu.h"

/*----------------------------------------------------------------------
 * CmdASCII
 *
 *      Change to ascii transfer mode.
 *      Equivalent to "mode ascii".
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdASCII()
{
  modeType = MODE_ASCII;
  printf("mode is now ascii.\n");
}

/*----------------------------------------------------------------------
 * CmdAutoadapt
 *
 *      Toggle automatic adaption on/off.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdAutoadapt()
{
  int temp;

  if((temp = CSOnOff()) != -1) {
    renegotiate = temp;
  }
  printf("autoadapt mode is now %s.\n", renegotiate ? "on" : "off");
}

/*----------------------------------------------------------------------
 * CmdBaudrate
 *
 *      Set estimated channel transfer-speed.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdBaudrate()
{
  long newBaudRate;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("an estimated data-transfer-rate must be specified.");
  } else {
    newBaudRate = atol( tempToken) ;
    if((newBaudRate < 42) || (newBaudRate > 1000000)) {
      CSErrCmd("valid baudrates are [42..1000000].");
    } else {
      /* If value is invalid, next line will exit function call */
      CSCheckBurstRate("baudrate", blockSize, burstSize, newBaudRate);

      /* baudrate is set OK according to calulation of burstRate */
      baudRate = newBaudRate;
      printf("baudrate set to %ld\n", baudRate);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdBinary
 *
 *      Change to binary transfer mode.
 *      Equivalent to "mode binary".
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdBinary()
{
  modeType = MODE_BINARY;
  printf("mode is now binary.\n");
}

/*----------------------------------------------------------------------
 * CmdBlocksize
 *
 *      Change current data-block-size.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdBlocksize()
{
  int newBlockSize;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("a blocksize must be supplied.");
  } else {
    newBlockSize = (atoi( tempToken ) / 4)*4;
    if((newBlockSize < 16) || (newBlockSize > 1900)) {
      CSErrCmd("valid blocksizes are [16..1900].");
    } else {
      /* If value is invalid, next line will exit function call */
      CSCheckBurstRate("blocksize", newBlockSize, burstSize, baudRate);

      blockSize = newBlockSize;
      printf("blocksize set to %d.\n", blockSize);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdBufferblock
 *
 *      Change no of data-blocks per buffer.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdBufferblock()
{
  int newBufferSize;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("number of bufferblocks must be supplied.");
  } else {
    newBufferSize = atoi( tempToken );
    if ((newBufferSize < 1) || (newBufferSize > 200)) {
      CSErrCmd("valid number of blocks per buffer are [1..200].");
    } else { 
      bufferSize = newBufferSize;
      printf("bufferblock set to %d.\n", bufferSize);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdBurstsize
 *
 *      Change size of data-packet-burst.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdBurstsize()
{
  int newBurstSize;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("a burstsize must be specified.");
  } else {
    newBurstSize = atoi( tempToken );
    if ((newBurstSize < 1) || (newBurstSize > 20)) {
      CSErrCmd("valid burstsizes are [1..20].");
    } else {
      /* If value is invalid, next line will exit function call */
      CSCheckBurstRate("burstsize", blockSize, newBurstSize, baudRate);

      burstSize = newBurstSize;
      printf("burstsize set to %d\n", burstSize);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdConnect
 *
 * connect <hostname> [ <port> ]
 *
 *      Set the hostname and optional port number for future transfers.
 *      The port is the well-known port number of the etftp server on
 *      the other system.  This should default to the value specified in
 *      /etc/services when etftp becomes a standard. Default port is 1818.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdConnect()
{
  register int val;
  
  if (CSGetToken( hostName ) == NULL) {
    CSErrCmd("hostname is missing.");
    return;
  }
  if (gethostbyname( hostName ) == NULL) {
    CSErrCmd("hostname not recognized.");
    return;
  }
  connected = 1;
  if (CSGetToken( tempToken ) == NULL) {
    printf("Connected to %s.\n", hostName);
    return;
  }
  val = atoi( tempToken );
  if (val <= 0) {
    CSErrCmd("invalid port number.");
  }
  remotePort = val;
    printf("Connected to %s on port %d.\n", hostName, remotePort);
}

/*----------------------------------------------------------------------
 * CmdExit
 *
 *      Terminate client.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdExit()
{
  printf("End of line.\n");
  exit(0);
  if (traceFlag) {
    CLUStop();			/* close logfile  */
  }
}

/*----------------------------------------------------------------------
 * CmdGet
 *
 * get <remotefilename> <localfilename>
 *
 *      Note that the <remotefilename> may be of the form <host>:<filename>
 *      to specify the host also.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdGet()
{
  char remfname[MAXFILENAME], locfname[MAXFILENAME];

  if (CSGetToken( remfname ) == NULL) {
    CSErrCmd("the remote filename must be specified.");
  }
  if (CSGetToken( locfname ) == NULL) {
    CSErrCmd("the local filename must be specified.");
  }
  if (strchr(locfname, ':') != NULL) {
    CSErrCmd("one can't have 'host:' in local filename.");
  }
  
  CSStripHost(remfname, hostName);  /* check for "host:" and process */
  if (hostName[0] == 0) {
    CSErrCmd("no host has been specified.");
  }
  DoGet(remfname, locfname);
}

/*----------------------------------------------------------------------
 * CmdHelp
 *
 *      Display a command-list with short descriptions.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdHelp()
{
  register int i;

  for (i = 0; i < nCmds; i++) {
    printf("  %s\n", commands[i].cmdDesc);
  }
}

/*----------------------------------------------------------------------
 * CmdLocalport
 *
 *      Set portnumber for client to listen on.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdLocalport()
{
  int newLocalPort;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("a local port number must be specified.");
  } else {
    newLocalPort=atoi( tempToken );
    if((newLocalPort < 0) || (newLocalPort > 65535)) {
      CSErrCmd("valid portnumbers are [0..65535].");
    } else {
      localPort = newLocalPort;
      printf("localport set to %d.\n", localPort);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdMode
 *
 *      Set the mode for file transfers.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdMode()
{
  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("a mode type must be specified.");
  } else {
    if (strcmp(tempToken, "ascii") == 0) {
      modeType = MODE_ASCII;
    } else if (strcmp(tempToken, "binary") == 0) {
      modeType = MODE_BINARY;
    } else {
      CSErrCmd("mode must be 'ascii' or 'binary'");
    }
  }
}

/*----------------------------------------------------------------------
 * CmdMultibuff
 *
 *      Set the number of active buffers (1 or 2 in semiduplex NW).
 *      THIS FEATURE IS NOT IMPLEMENTED.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdMultibuff()
{
  int newMultiBuff;

  if (CSGetToken(tempToken) == NULL) {
    CSErrCmd("number of buffers must be specified.");
  } else {
    newMultiBuff = atoi( tempToken );
    if((newMultiBuff !=1 ) && (newMultiBuff != 2)) {
      CSErrCmd("valid number of buffers are [1,2].");
    } else {
      multiBuff = newMultiBuff;
      printf("no of buffers set to %d.\n", multiBuff);
      printf("NOTE: multibuff handling is not implemented.\n");
    }
  }
}

/*----------------------------------------------------------------------
 * CmdPut
 *
 * put <localfilename> <remotefilename>
 *
 *      Note that the <remotefilename> may be of the form <host>:<filename>
 *      to specify the host also.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdPut()
{
  char remfname[MAXFILENAME], locfname[MAXFILENAME];

  if (CSGetToken( locfname ) == NULL) {
    CSErrCmd("the local filename must be specified.");
  }
  if (CSGetToken( remfname ) == NULL) {
    CSErrCmd("the remote filename must be specified.");
  }
  if (strchr(locfname, ':') != NULL) {
    CSErrCmd("one can't have 'host:' in local filename.");
  }
  
  CSStripHost(remfname, hostName);  /* check for "host:" and process */
  if (hostName[0] == 0) {
    CSErrCmd("no host has been specified.");
  }
  DoPut(remfname, locfname);
}

/*----------------------------------------------------------------------
 * CmdRadiodelay
 *
 *      Set the estimated turnover time for the semiduplex-channel.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdRadiodelay()
{
  unsigned short newRadioDelay;

  if (CSGetToken( tempToken ) == NULL) {
    CSErrCmd("a time in seconds must be given.");
  } else {
    newRadioDelay = atoi( tempToken );
    if((newRadioDelay > 32767)) {
      /* according to an example in RFC 1996 (page 15) an upper limit
         of at least 32767 seconds suffice (deep space probe ) */
      CSErrCmd("valid radiodelays are [0..32767].");
    } else {
      radioDelay = newRadioDelay;
      printf("radiodelay set to %d seconds.\n", radioDelay);
    }
  }
}

/*----------------------------------------------------------------------
 * CmdStatus
 *
 *      Show current status.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdStatus()
{
  printf("  Running ETFTP version " ETFTP_VERSION "\n");

  if (connected) {
    printf("  Connected to %s", hostName);
    if (remotePort != 1818) {
      printf(" on port = %d.\n", remotePort);
    } else {
      printf(".\n");
    }
  } else {
    printf("  Not connected\n");
  }
  if (localPort != 1818) {
    printf("  Listening on port = %d\n", localPort);
  }
  printf("  Transfer mode is ");
  switch ( modeType ) {
  case MODE_ASCII:
    printf("ascii.\n");
    break;
  case MODE_BINARY:
    printf("binary.\n");
    break;
  default:
    printf("unknown modetype.\n");
  }
  printf("  verbose = %s\n", verboseFlag ? "on" : "off");
  printf("  hash = %s\n", hashFlag ? "on" : "off");
  printf("  trace = %s\n", traceFlag ? "on" : "off");
  printf("  autoadapt = %s\n", renegotiate ? "on" : "off");
  printf("  radiodelay = %d seconds\n", radioDelay);
  printf("  multibuff = %d buffers\n", multiBuff);
  printf("  blocksize = %d\n", blockSize);
  printf("  bufferblock = %d\n", bufferSize);
  printf("  burstsize = %d blocks\n", burstSize);
  printf("  baudrate = %ld bps\n", baudRate);
}

/*----------------------------------------------------------------------
 * CmdTrace
 *
 *      Toggle analysis mode.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdTrace()
{
  int temp;

  if((temp = CSOnOff()) != -1) {
    traceFlag = temp;
  }
  printf("trace mode is now %s.\n", traceFlag ? "on" : "off");
}

/*----------------------------------------------------------------------
 * CmdVerbose
 *
 *      Toggle verbose mode.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdVerbose()
{
  int temp;

  if((temp = CSOnOff()) != -1) {
    verboseFlag = temp;
  }
  printf("verbose mode is now %s.\n", verboseFlag ? "on" : "off");
}

/*----------------------------------------------------------------------
 * CmdHash
 *
 *      Toggle hash mode.
 *
 * Results:
 *      None.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

void CmdHash()
{
  int temp;

  if((temp = CSOnOff()) != -1) {
    hashFlag = temp;
  }
  printf("hash mode is now %s.\n", hashFlag ? "on" : "off");
}

/*       E N D    O F     L I N E       */


