/*
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:   file.c
 * Class:               File
 * Created:     0.1, 970418, Torbjrn Frostemark
 * Revised:     1.1, 980219, Torbjrn Frostemark
 *
 * Description: Routines to open/close/read/write the local file.
 *              For "binary" (octet) transmissions, we use the UNIX
 *              open/read/write system calls (or their equivalent).
 *              For "ascii" (netascii) transmissions, we use the UNIX
 *              standard i/o routines fopen/getc/putc (or their equivalent).
 * Remarks:     -
 *
 * Private Metods:      -
 *
 * Private Attributes:  int lastCR;       flag - true if last char was CR
 *                      int nextChar;
 *----------------------------------------------------------------------
 */
        
#include "file.h"
#include "clu.h"
#include <stdio.h>
        
/*
 * Attributes to the File object.
 */
        
static int lastCR = 0;          /* 1 if last character was a carriage-return */
static int nextChar = 0;

/*----------------------------------------------------------------------
 * FileCreate
 *
 *      Create local file on file system.
 *
 * Results:
 *      Returns a FILE pointer, or NULL on error.
 *
 *
 * Side effects:
 *      Creates a new file in the file system.
 *----------------------------------------------------------------------
 */

FILE *FileCreate(fileName)
     char *fileName;            /* The name of the file to be created */
{
  register FILE *fd;
  
  if ( (fd = fopen(fileName, "w")) == NULL) {
    return (FILE *)NULL;	/* if file cannot be opened */
  }

  /*printf("FileCreate: Creating file\n");*/
  lastCR = 0;			/* for FileWrite() */
  nextChar = -1;		/* for FileRead() */
  
  return fd;
}

/*----------------------------------------------------------------------
 * FileOpen
 *
 *      Open the local file for reading.
 *
 * Results:
 *      Return a FILE ponter, or NULL on error.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

FILE *FileOpen(fileName)
     char *fileName;
{
  register FILE *fd;
  
  if ( (fd = fopen(fileName, "r")) == NULL) {
    return (FILE *)NULL;	/* if file cannot be opened */
  }

  /*printf("FileOpen: Open file\n");*/
  lastCR = 0;			/* for FileWrite() */
  nextChar = -1;		/* for FileRead() */
  
  return fd;
}

/*----------------------------------------------------------------------
 * FileClose
 *
 *      Close the local file.
 *
 * Results:
 *      Returns 1 if OK, 0 on error;
 *      This causes the standard i/o system to flush its buffers for
 *      this file.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */
        
int FileClose(fd)
     FILE *fd;
{
  if (lastCR) {
    DEBUG0("FileError: final character was a CR.");
    return 0;
  }
  if (nextChar >= 0) {
    DEBUG0("FileError: nextChar >= 0.");
    return 0;
  }
  if (fclose(fd) == EOF) {
    DEBUG0("FileError: fclose failed.");
    return 0;
  }
  /*printf("FileClose: File closed\n");*/
  return 1;
}

/*----------------------------------------------------------------------
 * FileRead
 *
 *      Read data from the local file.
 *
 * Results:
 *      Returns the number of bytes read to buffer (between 1 and maxnBytes,
 *      inclusive) or 0 on EOF.
 *
 * Side effects:
 *      Here is where we handle any conversion between the file's mode
 *      on the local system and the network mode.
 *----------------------------------------------------------------------
 */
int FileRead(fd, buffer, maxnBytes, mode)
     FILE           *fd;
     register char  *buffer;
     register int   maxnBytes;
     int            mode;	/* MODE_BINARY or MODE_ASCII */
{
  int   c, byteCount;
  
  switch (mode) {
  case MODE_BINARY:
    for(byteCount = 0; byteCount < maxnBytes; byteCount++) {
      buffer[byteCount] = fgetc(fd);
      if(ferror(fd) || feof(fd)) {
	return byteCount;
      }
    }
    return byteCount;		/* will be 0 on EOF */      
  case MODE_ASCII:

    /*
     * For files that are transferred in netascii, we must
     * perform the reverse conversions that FileWrite() does.
     * Note that we have to use the attribute "nextChar" to
     * remember if the next character to output is a linefeed
     * or a null, since the second byte of a 2-byte sequence
     * may not fit in the current buffer, and may have to go
     * as the first byte of the next buffer (i.e., we have to
     * remember this fact from one call to the next).
     */

    for (byteCount = 0 ; byteCount < maxnBytes ; byteCount++) {
      if (nextChar >= 0) {
	*buffer++ = nextChar;
	nextChar = -1;
	continue;
      }
      switch ( c = getc(fd) ) {
      case EOF:			/* EOF return means eof or error */
	if (ferror(fd)) {
	  DEBUG0("FileError: Read error from getc on local file.");
	}
	return byteCount;	/* No real byte was read */
	break;
      case '\n':
	c = '\r';		/* newline -> CR,LF */
	nextChar = '\n';
	break;
      case '\r':
	nextChar = '\0';        /* CR -> CR,NULL */
	break;
      default:
	nextChar = -1;
      }
                  
      *buffer++ = c;		/* Write char to buffer */
    }
    /*printf("FileRead: Read %d from file\n", byteCount);*/
    return byteCount;
  default:
    DEBUG0("FileError: Unknown MODE value.");
  }
  return 0;			/* Nothing read */
}

/*----------------------------------------------------------------------
 * FileWrite
 *
 *      Write data to the local file.
 *
 * Results:
 *      Returns bytes written from buffer, or -1 on error.
 *
 *
 * Side effects:
 *      Here is where we handle any conversion between the mode of the
 *      file on the network and the local system's conventions.
 *----------------------------------------------------------------------
 */
        
int FileWrite(fd, buffer, nbytes, mode)
     FILE *fd;
     register char *buffer;
     register int  nbytes;
     int           mode;
{
  register int c, i, bytesWritten;

  switch (mode) {
  case MODE_BINARY:     /* For binary mode files, no conversion is required. */
    for(i = 0; i < nbytes; i++) {
      if((fputc(buffer[i], fd) == EOF) && ferror(fd)) {
	break;
      }
    }
    if (i != nbytes) {
      DEBUG2("write error to local file, %d outof %d bytes written",i,nbytes);
      return -1;
    }
    bytesWritten = nbytes;
    break;
                
  case MODE_ASCII:

    /*
     * For files that are transferred in netascii, we must
     * perform the following conversions:
     *
     *      CR,LF                            ->  newline = '\n'
     *      CR,NULL                  ->  CR          = '\r'
     *      CR,anything_else  ->        undefined (we don't allow this)
     *
     * Note that we have to use the global "lastCR" to remember
     * if the last character was a carriage-return or not,
     * since if the last character of a buffer is a CR, we have
     * to remember that when we're called for the next buffer.
     */

    bytesWritten = 0;
    for (i = 0; i < nbytes; i++) {
      c = *buffer++;
      if (lastCR) {
	switch (c) {
	case '\n':
	  break;
	case '\0':
	  c = '\r';
	  break;
	default:
	  DEBUG1("CR followed by 0x%02x", c);
	  return 0;
	}
	lastCR = 0;
      } else if (c == '\r') {
	lastCR = 1;
	continue;		/* get next character */
      }
      bytesWritten++;
      if (putc(c, fd) == EOF) {
	DEBUG0("FileError: Write error from putc to local file.");
	return 0;
      }
    }
    break;
  default:
    DEBUG0("FileError: unknown MODE value.");
    return -1;
  }
  fflush(fd);
  return bytesWritten;
}

/*----------------------------------------------------------------------
 * FileCount
 *
 *      Calculate size of file converted to netascii.
 *
 * Results:
 *      Size in bytes or -1 if FileOpen fails.
 *
 * Side effects:
 *      None.
 *----------------------------------------------------------------------
 */

#define SCNBUFSZ 512

int FileCount(fileName, mode)
     char *fileName;
     int mode;
{
  FILE *fd;
  int byteCount, bytesRead;
  char *buf;
  buf = malloc(SCNBUFSZ);
  if((fd = FileOpen(fileName)) == NULL) {
    return -1;
  }
  byteCount = 0;
  for(; (bytesRead = FileRead(fd, buf, SCNBUFSZ, mode)) > 0;) {
    byteCount += bytesRead;
  }
  free(buf);
  FileClose(fd);
  return byteCount;
}
        

/*   E N D    O F    L I N E    */        
        
        
        
        
        
        
        
        
