/* 
 * Copyright (C) 1997, 1998 Defence Research Establishment, Dept. of Comm.
 * Systems, FOA 72, Sweden.
 * Currently maintained by Christian Jnsson FOA 72 <chj@lin.foa.se>.
 *
 * This file is part of ETFTP version 1.1.3.
 *
 *   ETFTP is free software; you can redistribute it and/or modify it
 *   under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2, or (at your option)
 *   any later version.
 *
 *   ETFTP is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with ETFTP; see the file GPL.  If not, write to the
 *   Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 *   Boston, MA 02111-1307, USA.
 *
 * Code:
 *
 *----------------------------------------------------------------------
 * Filenamne:	rtt.c
 * Class:	RTT
 * Created:	0.1, 970506, Per-Erik Forssn
 * Revised:	1.1, 980219, Torbjrn Frostemark
 * 
 * Description: Timer routines for round-trip timing of datagrams.
 * Remarks:	-
 *
 * Private Metods:	-
 *
 * Private Attributes:	-
 *----------------------------------------------------------------------
 */

#include "rtt.h"
#include "etftptime.h"
#include <unistd.h>

/*
 * Linear backoff value in seconds. To be multiplied with 
 * rtt_nrexmt = 0, 1, 2 .... This is the number  of times
 * a packet has been retransmitted.
 */

#define LIN_BACKOFF 5

/*----------------------------------------------------------------------
 * RTTInit
 *
 * 	Initializes an RTT structure.
 * 	This function is called before the first packet is transmitted.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

void RTTInit(ptr)
     register RTTStruct	*ptr;	/* ptr to caller's structure */
{
  int trt, frd;

  ptr->rtt = 0;			/* round trip time */ 

  trt = 2*ptr->rtt;		/* twice the round-trip time */
  frd = 4*radioDelay;		/* four times the radio delay */

  ptr->wait = (trt > frd ? trt : frd) + LIN_BACKOFF;
  /*  printf("RTTInit: Setting wait = %d\n", ptr->wait);*/
}

/*----------------------------------------------------------------------
 * RTTNewPack
 *
 *	Called before each new packet is transmitted on
 *	a "connection."  Resets retransmit counter and
 *	calculates new values for timeout counters.
 *
 * Results:
 *	Updates timeout counters.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

void RTTNewPack(ptr, packNum)
     register RTTStruct	*ptr;	/* ptr to caller's structure */
     register int packNum;	/* DATA packet number */
{
  int trt, frd;

  trt = 2*rtTimer;	/* twice the tound-trip time */
  frd = 4*radioDelay;	/* four times the radio delay */

  ptr->wait = (trt > frd ? trt : frd) + LIN_BACKOFF;
  /*  printf("RTTNewPack: Setting wait = %d\n", ptr->wait);*/
  ptr->rtt_nrexmt = 0;

  /*  if ((pkRecv == PT_DATA) || (pkRecv == PT_LDATA)) {*/
  if (packNum >= 0) {
    /* Receiving case */
    /*    printf("RTTNewPack: bufferSize = %d, packnum = %d, burstRate = %d, burstSize = %d\n",
	  bufferSize, packNum, burstRate, burstSize);*/
    /*    ptr->alarmTimeout = (bufferSize - packNum)*burstRate/burstSize/1000 + ptr->wait; */
    ptr->alarmTimeout = (int)                          /* Timer fixed by ChJ */
      ((float)(bufferSize - packNum)                   /* Timer fixed by ChJ */
       * burstRate/burstSize + ptr->wait);
    /*    printf("RTTNewPack(recv): Setting alarmTimeout = %d (wait = %d)\n", ptr->alarmTimeout,
	  ptr->wait);*/
  } else {
    /* Normal case */
    ptr->alarmTimeout = ptr->wait;
    /*    printf("RTTNewPack(send): Setting alarmTimeout = %d (wait = %d)\n", ptr->alarmTimeout,
	  ptr->wait);*/
  }
}

/*----------------------------------------------------------------------
 * RTTUpdate
 *
 *	Called before a packet has been received.
 *	Update transmit timeout counters efter a resend.
 *
 * Results:
 *	Returns current transmit timeout value and updates counters
 *	during resend phase.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

int RTTUpdate(ptr, packNum)
     register RTTStruct	*ptr;	/* ptr to caller's structure */
     register int packNum;	/* DATA packet number */
{
  int trt, frd;

  /*
   * These are some special cases that has to be treated a little differently
   * since the connection uptime may take a little longer than it should.
   */

  if (pkSent == PT_OPEN)
    /* if OPEN was lost, wait a little longer */
    return 100;
  
  if (pkSent == PT_RESPONSE)
    /* if RESPONSE was lost, wait a little longer */
    return 100;

  /*
   * End of special cases.
   */

  if (ptr->rtt_nrexmt > 0) {
    /* This is a resend */

    trt = 2*rtTimer;
    frd = 4*radioDelay;

    ptr->wait = (trt > frd ? trt : frd) + LIN_BACKOFF*(ptr->rtt_nrexmt + 1);
    /*    printf("RTTUpdate: Setting wait = %d\n", ptr->wait);*/

    if (packNum >= 0) {
      /* Receiving case */
      /*      printf("RTTUpdate: bufferSize = %d, packnum = %d, burstRate = %d, burstSize = %d\n",
	      bufferSize, packNum, burstRate, burstSize);*/
      /*      ptr->alarmTimeout = (bufferSize - packNum)*burstRate/burstSize/1000 + ptr->wait; */
      ptr->alarmTimeout = (int)                        /* Timer fixed by ChJ */
	((float)(bufferSize - packNum)                 /* Timer fixed by ChJ */
	 * burstRate/burstSize + ptr->wait);
      /*      printf("RTTUpdate(recv): Setting alarmTimeout = %d (wait = %d)\n", ptr->alarmTimeout,
	      ptr->wait);*/
    } else {
      /* Normal case */
      ptr->alarmTimeout = ptr->wait;
      /*      printf("RTTUpdate(send): Setting alarmTimeout = %d (wait = %d)\n", ptr->alarmTimeout,
	      ptr->wait);*/
    }
  }
  return(ptr->alarmTimeout);
}

/*----------------------------------------------------------------------
 * RTTTimeout
 *
 *	Called after a timeout has occurred.  Tells you
 *	whether you should retransmit again, or give up.
 * 	A timeout has occurred.
 * 	This function should be called right after the timeout alarm occurs.
 *
 * Results:
 *	Return -1 if it's time to give up, else return 0.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

int RTTTimeout(ptr)
     register RTTStruct	*ptr;	/* ptr to caller's structure */
{
  if (ptr->rtt_nrexmt >= RTT_DEATH) {
    return -1;		/* time to give up for this packet */
  }
  ptr->rtt_nrexmt++;	/* increase number of resends */
  return 0;
}

/*----------------------------------------------------------------------
 * RTTMsleep
 *
 * 	Sleeps in milliseconds instead of UNIX standard seconds.
 *
 * Results:
 *	None.
 *
 * Side effects:
 *	None.
 *----------------------------------------------------------------------
 */

#ifdef LINUX_ETFTP
void RTTMsleep(mSecs)
     int mSecs;
{
  printf("RTTMsleep: mSec = %d\n", mSecs);
  usleep(1000*mSecs);
}
#else
void RTTMsleep(mSecs)
int mSecs;
{
  struct timeval tV;
  
  tV.tv_sec=mSecs/MILLISEC;
  tV.tv_usec=mSecs%MICROSEC;
  select(0,NULL,NULL,NULL,&tV);
}
#endif /* LINUX_ETFTP */

/*     E N D   O F   L I N E    */









